// University of Illinois/NCSA
// Open Source License
//
// Copyright (c) 2013-2015, Advanced Micro Devices, Inc.
// All rights reserved.
//
// Developed by:
//
//     HSA Team
//
//     Advanced Micro Devices, Inc
//
//     www.amd.com
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal with
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
//     * Redistributions of source code must retain the above copyright notice,
//       this list of conditions and the following disclaimers.
//
//     * Redistributions in binary form must reproduce the above copyright notice,
//       this list of conditions and the following disclaimers in the
//       documentation and/or other materials provided with the distribution.
//
//     * Neither the names of the LLVM Team, University of Illinois at
//       Urbana-Champaign, nor the names of its contributors may be used to
//       endorse or promote products derived from this Software without specific
//       prior written permission.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE
// SOFTWARE.

class InstValidator : public PropValidator
{
private:

    // Autogenerated accessors for BRIG properties
    #include "HSAILBrigPropsFastAcc_gen.hpp"

public:
    InstValidator(unsigned model, unsigned profile) : PropValidator(model, profile) {}
    void validateInst(Inst inst) const;

public:
    unsigned getOperand0Attr(Inst inst) const;

private:

public:
    unsigned getOperand1Attr(Inst inst) const;

private:

public:
    unsigned getOperand2Attr(Inst inst) const;

private:

public:
    unsigned getOperand3Attr(Inst inst) const;

private:

public:
    unsigned getOperand4Attr(Inst inst) const;

private:

public:
    unsigned getOperand5Attr(Inst inst) const;

private:

public:
    unsigned getOperandAttr(Inst inst, unsigned operandIdx) const
    {
        switch(operandIdx)
        {
        case 0: return getOperand0Attr(inst);
        case 1: return getOperand1Attr(inst);
        case 2: return getOperand2Attr(inst);
        case 3: return getOperand3Attr(inst);
        case 4: return getOperand4Attr(inst);
        case 5: return getOperand5Attr(inst);
        default:
            assert(false);
            return OPERAND_ATTR_INVALID;
        }
    }


private:
    static unsigned OPERAND_VALUES_REG_IMM[];
    static unsigned OPERAND_VALUES_NULL[];
    static unsigned OPERAND_VALUES_REG[];
    static unsigned TYPE_VALUES_U32[];

private:
    static bool check_type_values_u32(unsigned val);

private:
    template<class T> bool req_d0_s1(T inst) const;
    template<class T> bool req_d0_s1_s2(T inst) const;
    template<class T> bool req_d0_s1_s2_s3(T inst) const;
    template<class T> bool req_dg_group(T inst) const;

}; // class InstValidator

unsigned InstValidator::OPERAND_VALUES_REG_IMM[] = {
    OPERAND_VAL_IMM,
    OPERAND_VAL_REG
};

unsigned InstValidator::OPERAND_VALUES_NULL[] = {
    OPERAND_VAL_NULL
};

unsigned InstValidator::OPERAND_VALUES_REG[] = {
    OPERAND_VAL_REG
};

unsigned InstValidator::TYPE_VALUES_U32[] = {
    BRIG_TYPE_U32
};

bool InstValidator::check_type_values_u32(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_U32:
        return true;
    default:
        return false;
    }
}

//================================================================================
//  Req d0_s1 = {
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_d0_s1(T inst) const
{
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req d0_s1_s2 = {
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg, imm;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_d0_s1_s2(T inst) const
{
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req d0_s1_s2_s3 = {
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg, imm;
//      s3 = reg, imm;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_d0_s1_s2_s3(T inst) const
{
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req dg_group = {
//      type = u32;
//      d0 = reg;
//      s1 = null;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_dg_group(T inst) const
{
    if (!check_type_values_u32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_U32, sizeof(TYPE_VALUES_U32) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

void InstValidator::validateInst(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_DG_GROUPSTATICSIZE):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_dg_group<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_DG_GROUPTOTALSIZE):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_dg_group<InstBasic>(i);
            break;
        }
        default:
            validate(inst, false, "Invalid instruction opcode");
            break;
    }
} // InstValidator::validateInst

unsigned InstValidator::getOperand0Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_DG_GROUPSTATICSIZE): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_DG_GROUPTOTALSIZE): return OPERAND_ATTR_DTYPE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand0Attr

unsigned InstValidator::getOperand1Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_DG_GROUPSTATICSIZE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_DG_GROUPTOTALSIZE): return OPERAND_ATTR_NONE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand1Attr

unsigned InstValidator::getOperand2Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_DG_GROUPSTATICSIZE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_DG_GROUPTOTALSIZE): return OPERAND_ATTR_NONE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand2Attr

unsigned InstValidator::getOperand3Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_DG_GROUPSTATICSIZE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_DG_GROUPTOTALSIZE): return OPERAND_ATTR_NONE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand3Attr

unsigned InstValidator::getOperand4Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_DG_GROUPSTATICSIZE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_DG_GROUPTOTALSIZE): return OPERAND_ATTR_NONE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand4Attr

unsigned InstValidator::getOperand5Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_DG_GROUPSTATICSIZE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_DG_GROUPTOTALSIZE): return OPERAND_ATTR_NONE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand5Attr

