// University of Illinois/NCSA
// Open Source License
//
// Copyright (c) 2013-2015, Advanced Micro Devices, Inc.
// All rights reserved.
//
// Developed by:
//
//     HSA Team
//
//     Advanced Micro Devices, Inc
//
//     www.amd.com
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal with
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
//     * Redistributions of source code must retain the above copyright notice,
//       this list of conditions and the following disclaimers.
//
//     * Redistributions in binary form must reproduce the above copyright notice,
//       this list of conditions and the following disclaimers in the
//       documentation and/or other materials provided with the distribution.
//
//     * Neither the names of the LLVM Team, University of Illinois at
//       Urbana-Champaign, nor the names of its contributors may be used to
//       endorse or promote products derived from this Software without specific
//       prior written permission.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE
// SOFTWARE.

class InstValidator : public PropValidator
{
private:

    // Autogenerated accessors for BRIG properties
    #include "HSAILBrigPropsFastAcc_gen.hpp"

public:
    InstValidator(unsigned model, unsigned profile) : PropValidator(model, profile) {}
    void validateInst(Inst inst) const;

public:
    unsigned getOperand0Attr(Inst inst) const;

private:

public:
    unsigned getOperand1Attr(Inst inst) const;

private:

public:
    unsigned getOperand2Attr(Inst inst) const;

private:

public:
    unsigned getOperand3Attr(Inst inst) const;

private:

public:
    unsigned getOperand4Attr(Inst inst) const;

private:

public:
    unsigned getOperand5Attr(Inst inst) const;

private:

public:
    unsigned getOperandAttr(Inst inst, unsigned operandIdx) const
    {
        switch(operandIdx)
        {
        case 0: return getOperand0Attr(inst);
        case 1: return getOperand1Attr(inst);
        case 2: return getOperand2Attr(inst);
        case 3: return getOperand3Attr(inst);
        case 4: return getOperand4Attr(inst);
        case 5: return getOperand5Attr(inst);
        default:
            assert(false);
            return OPERAND_ATTR_INVALID;
        }
    }


private:
    static unsigned EQCLASS_VALUES_ANY[];
    static unsigned GEOMETRY_VALUES_1D[];
    static unsigned GEOMETRY_VALUES_ANY[];
    static unsigned GEOMETRY_VALUES_1D_2D_3D_1DA_2DA_2DDEPTH_2DADEPTH[];
    static unsigned GEOMETRY_VALUES_1D_1DB[];
    static unsigned GEOMETRY_VALUES_2D_1DA[];
    static unsigned GEOMETRY_VALUES_1DA_2DA_2DADEPTH[];
    static unsigned GEOMETRY_VALUES_2D_3D_2DA_2DDEPTH_2DADEPTH[];
    static unsigned GEOMETRY_VALUES_3D_2DA[];
    static unsigned GEOMETRY_VALUES_2DADEPTH[];
    static unsigned GEOMETRY_VALUES_2DDEPTH[];
    static unsigned GEOMETRY_VALUES_3D[];
    static unsigned IPROP_VALUES_ARRAY[];
    static unsigned IPROP_VALUES_ANY[];
    static unsigned IPROP_VALUES_WIDTH_CHANNELTYPE_CHANNELORDER[];
    static unsigned IPROP_VALUES_DEPTH[];
    static unsigned IPROP_VALUES_HEIGHT[];
    static unsigned OPERAND_VALUES_REG_IMM[];
    static unsigned OPERAND_VALUES_NULL[];
    static unsigned OPERAND_VALUES_REG[];
    static unsigned OPERAND_VALUES_VEC2CTYPE[];
    static unsigned OPERAND_VALUES_VEC3CTYPE[];
    static unsigned OPERAND_VALUES_VEC4[];
    static unsigned SPROP_VALUES_ANY[];
    static unsigned TYPE_VALUES_S32_U32_F32_F16[];
    static unsigned TYPE_VALUES_S32_F32[];
    static unsigned TYPE_VALUES_NONE[];
    static unsigned TYPE_VALUES_ROIMG[];
    static unsigned TYPE_VALUES_ROIMG_RWIMG[];
    static unsigned TYPE_VALUES_ROIMG_RWIMG_WOIMG[];
    static unsigned TYPE_VALUES_WOIMG_RWIMG[];
    static unsigned TYPE_VALUES_U32[];

private:
    static bool check_geometry_values_1d(unsigned val);
    static bool check_geometry_values_any(unsigned val);
    static bool check_geometry_values_1d_2d_3d_1da_2da_2ddepth_2dadepth(unsigned val);
    static bool check_geometry_values_1d_1db(unsigned val);
    static bool check_geometry_values_2d_1da(unsigned val);
    static bool check_geometry_values_1da_2da_2dadepth(unsigned val);
    static bool check_geometry_values_2d_3d_2da_2ddepth_2dadepth(unsigned val);
    static bool check_geometry_values_3d_2da(unsigned val);
    static bool check_geometry_values_2dadepth(unsigned val);
    static bool check_geometry_values_2ddepth(unsigned val);
    static bool check_geometry_values_3d(unsigned val);
    static bool check_iprop_values_array(unsigned val);
    static bool check_iprop_values_any(unsigned val);
    static bool check_iprop_values_width_channeltype_channelorder(unsigned val);
    static bool check_iprop_values_depth(unsigned val);
    static bool check_iprop_values_height(unsigned val);
    static bool check_sprop_values_any(unsigned val);
    static bool check_type_values_s32_u32_f32_f16(unsigned val);
    static bool check_type_values_s32_f32(unsigned val);
    static bool check_type_values_none(unsigned val);
    static bool check_type_values_roimg(unsigned val);
    static bool check_type_values_roimg_rwimg(unsigned val);
    static bool check_type_values_roimg_rwimg_woimg(unsigned val);
    static bool check_type_values_woimg_rwimg(unsigned val);
    static bool check_type_values_u32(unsigned val);

private:
    template<class T> bool req_d0_s1(T inst) const;
    template<class T> bool req_d0_s1_s2(T inst) const;
    template<class T> bool req_d0_s1_s2_s3(T inst) const;
    template<class T> bool req_imagefence(T inst) const;
    template<class T> bool req_ld_image(T inst) const;
    template<class T> bool req_ld_st_image(T inst) const;
    template<class T> bool req_queryimage(T inst) const;
    template<class T> bool req_querysampler(T inst) const;
    template<class T> bool req_rdimage(T inst) const;
    template<class T> bool req_st_image(T inst) const;

}; // class InstValidator

unsigned InstValidator::EQCLASS_VALUES_ANY[] = {
    EQCLASS_VAL_ANY
};

unsigned InstValidator::GEOMETRY_VALUES_1D[] = {
    BRIG_GEOMETRY_1D
};

unsigned InstValidator::GEOMETRY_VALUES_ANY[] = {
    BRIG_GEOMETRY_1D,
    BRIG_GEOMETRY_1DA,
    BRIG_GEOMETRY_1DB,
    BRIG_GEOMETRY_2D,
    BRIG_GEOMETRY_2DA,
    BRIG_GEOMETRY_2DADEPTH,
    BRIG_GEOMETRY_2DDEPTH,
    BRIG_GEOMETRY_3D
};

unsigned InstValidator::GEOMETRY_VALUES_1D_2D_3D_1DA_2DA_2DDEPTH_2DADEPTH[] = {
    BRIG_GEOMETRY_1D,
    BRIG_GEOMETRY_1DA,
    BRIG_GEOMETRY_2D,
    BRIG_GEOMETRY_2DA,
    BRIG_GEOMETRY_2DADEPTH,
    BRIG_GEOMETRY_2DDEPTH,
    BRIG_GEOMETRY_3D
};

unsigned InstValidator::GEOMETRY_VALUES_1D_1DB[] = {
    BRIG_GEOMETRY_1D,
    BRIG_GEOMETRY_1DB
};

unsigned InstValidator::GEOMETRY_VALUES_2D_1DA[] = {
    BRIG_GEOMETRY_1DA,
    BRIG_GEOMETRY_2D
};

unsigned InstValidator::GEOMETRY_VALUES_1DA_2DA_2DADEPTH[] = {
    BRIG_GEOMETRY_1DA,
    BRIG_GEOMETRY_2DA,
    BRIG_GEOMETRY_2DADEPTH
};

unsigned InstValidator::GEOMETRY_VALUES_2D_3D_2DA_2DDEPTH_2DADEPTH[] = {
    BRIG_GEOMETRY_2D,
    BRIG_GEOMETRY_2DA,
    BRIG_GEOMETRY_2DADEPTH,
    BRIG_GEOMETRY_2DDEPTH,
    BRIG_GEOMETRY_3D
};

unsigned InstValidator::GEOMETRY_VALUES_3D_2DA[] = {
    BRIG_GEOMETRY_2DA,
    BRIG_GEOMETRY_3D
};

unsigned InstValidator::GEOMETRY_VALUES_2DADEPTH[] = {
    BRIG_GEOMETRY_2DADEPTH
};

unsigned InstValidator::GEOMETRY_VALUES_2DDEPTH[] = {
    BRIG_GEOMETRY_2DDEPTH
};

unsigned InstValidator::GEOMETRY_VALUES_3D[] = {
    BRIG_GEOMETRY_3D
};

unsigned InstValidator::IPROP_VALUES_ARRAY[] = {
    BRIG_IMAGE_QUERY_ARRAY
};

unsigned InstValidator::IPROP_VALUES_ANY[] = {
    BRIG_IMAGE_QUERY_ARRAY,
    BRIG_IMAGE_QUERY_CHANNELORDER,
    BRIG_IMAGE_QUERY_CHANNELTYPE,
    BRIG_IMAGE_QUERY_DEPTH,
    BRIG_IMAGE_QUERY_HEIGHT,
    BRIG_IMAGE_QUERY_WIDTH
};

unsigned InstValidator::IPROP_VALUES_WIDTH_CHANNELTYPE_CHANNELORDER[] = {
    BRIG_IMAGE_QUERY_CHANNELORDER,
    BRIG_IMAGE_QUERY_CHANNELTYPE,
    BRIG_IMAGE_QUERY_WIDTH
};

unsigned InstValidator::IPROP_VALUES_DEPTH[] = {
    BRIG_IMAGE_QUERY_DEPTH
};

unsigned InstValidator::IPROP_VALUES_HEIGHT[] = {
    BRIG_IMAGE_QUERY_HEIGHT
};

unsigned InstValidator::OPERAND_VALUES_REG_IMM[] = {
    OPERAND_VAL_IMM,
    OPERAND_VAL_REG
};

unsigned InstValidator::OPERAND_VALUES_NULL[] = {
    OPERAND_VAL_NULL
};

unsigned InstValidator::OPERAND_VALUES_REG[] = {
    OPERAND_VAL_REG
};

unsigned InstValidator::OPERAND_VALUES_VEC2CTYPE[] = {
    OPERAND_VAL_VEC_2
};

unsigned InstValidator::OPERAND_VALUES_VEC3CTYPE[] = {
    OPERAND_VAL_VEC_3
};

unsigned InstValidator::OPERAND_VALUES_VEC4[] = {
    OPERAND_VAL_VEC_4
};

unsigned InstValidator::SPROP_VALUES_ANY[] = {
    BRIG_SAMPLER_QUERY_ADDRESSING,
    BRIG_SAMPLER_QUERY_COORD,
    BRIG_SAMPLER_QUERY_FILTER
};

unsigned InstValidator::TYPE_VALUES_S32_U32_F32_F16[] = {
    BRIG_TYPE_F16,
    BRIG_TYPE_F32,
    BRIG_TYPE_S32,
    BRIG_TYPE_U32
};

unsigned InstValidator::TYPE_VALUES_S32_F32[] = {
    BRIG_TYPE_F32,
    BRIG_TYPE_S32
};

unsigned InstValidator::TYPE_VALUES_NONE[] = {
    BRIG_TYPE_NONE
};

unsigned InstValidator::TYPE_VALUES_ROIMG[] = {
    BRIG_TYPE_ROIMG
};

unsigned InstValidator::TYPE_VALUES_ROIMG_RWIMG[] = {
    BRIG_TYPE_ROIMG,
    BRIG_TYPE_RWIMG
};

unsigned InstValidator::TYPE_VALUES_ROIMG_RWIMG_WOIMG[] = {
    BRIG_TYPE_ROIMG,
    BRIG_TYPE_RWIMG,
    BRIG_TYPE_WOIMG
};

unsigned InstValidator::TYPE_VALUES_WOIMG_RWIMG[] = {
    BRIG_TYPE_RWIMG,
    BRIG_TYPE_WOIMG
};

unsigned InstValidator::TYPE_VALUES_U32[] = {
    BRIG_TYPE_U32
};

bool InstValidator::check_geometry_values_1d(unsigned val)
{
    switch(val)
    {
    case BRIG_GEOMETRY_1D:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_geometry_values_any(unsigned val)
{
    switch(val)
    {
    case BRIG_GEOMETRY_1D:
    case BRIG_GEOMETRY_1DA:
    case BRIG_GEOMETRY_1DB:
    case BRIG_GEOMETRY_2D:
    case BRIG_GEOMETRY_2DA:
    case BRIG_GEOMETRY_2DADEPTH:
    case BRIG_GEOMETRY_2DDEPTH:
    case BRIG_GEOMETRY_3D:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_geometry_values_1d_2d_3d_1da_2da_2ddepth_2dadepth(unsigned val)
{
    switch(val)
    {
    case BRIG_GEOMETRY_1D:
    case BRIG_GEOMETRY_1DA:
    case BRIG_GEOMETRY_2D:
    case BRIG_GEOMETRY_2DA:
    case BRIG_GEOMETRY_2DADEPTH:
    case BRIG_GEOMETRY_2DDEPTH:
    case BRIG_GEOMETRY_3D:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_geometry_values_1d_1db(unsigned val)
{
    switch(val)
    {
    case BRIG_GEOMETRY_1D:
    case BRIG_GEOMETRY_1DB:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_geometry_values_2d_1da(unsigned val)
{
    switch(val)
    {
    case BRIG_GEOMETRY_1DA:
    case BRIG_GEOMETRY_2D:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_geometry_values_1da_2da_2dadepth(unsigned val)
{
    switch(val)
    {
    case BRIG_GEOMETRY_1DA:
    case BRIG_GEOMETRY_2DA:
    case BRIG_GEOMETRY_2DADEPTH:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_geometry_values_2d_3d_2da_2ddepth_2dadepth(unsigned val)
{
    switch(val)
    {
    case BRIG_GEOMETRY_2D:
    case BRIG_GEOMETRY_2DA:
    case BRIG_GEOMETRY_2DADEPTH:
    case BRIG_GEOMETRY_2DDEPTH:
    case BRIG_GEOMETRY_3D:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_geometry_values_3d_2da(unsigned val)
{
    switch(val)
    {
    case BRIG_GEOMETRY_2DA:
    case BRIG_GEOMETRY_3D:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_geometry_values_2dadepth(unsigned val)
{
    switch(val)
    {
    case BRIG_GEOMETRY_2DADEPTH:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_geometry_values_2ddepth(unsigned val)
{
    switch(val)
    {
    case BRIG_GEOMETRY_2DDEPTH:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_geometry_values_3d(unsigned val)
{
    switch(val)
    {
    case BRIG_GEOMETRY_3D:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_iprop_values_array(unsigned val)
{
    switch(val)
    {
    case BRIG_IMAGE_QUERY_ARRAY:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_iprop_values_any(unsigned val)
{
    switch(val)
    {
    case BRIG_IMAGE_QUERY_ARRAY:
    case BRIG_IMAGE_QUERY_CHANNELORDER:
    case BRIG_IMAGE_QUERY_CHANNELTYPE:
    case BRIG_IMAGE_QUERY_DEPTH:
    case BRIG_IMAGE_QUERY_HEIGHT:
    case BRIG_IMAGE_QUERY_WIDTH:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_iprop_values_width_channeltype_channelorder(unsigned val)
{
    switch(val)
    {
    case BRIG_IMAGE_QUERY_CHANNELORDER:
    case BRIG_IMAGE_QUERY_CHANNELTYPE:
    case BRIG_IMAGE_QUERY_WIDTH:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_iprop_values_depth(unsigned val)
{
    switch(val)
    {
    case BRIG_IMAGE_QUERY_DEPTH:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_iprop_values_height(unsigned val)
{
    switch(val)
    {
    case BRIG_IMAGE_QUERY_HEIGHT:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_sprop_values_any(unsigned val)
{
    switch(val)
    {
    case BRIG_SAMPLER_QUERY_ADDRESSING:
    case BRIG_SAMPLER_QUERY_COORD:
    case BRIG_SAMPLER_QUERY_FILTER:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_s32_u32_f32_f16(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_F16:
    case BRIG_TYPE_F32:
    case BRIG_TYPE_S32:
    case BRIG_TYPE_U32:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_s32_f32(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_F32:
    case BRIG_TYPE_S32:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_none(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_NONE:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_roimg(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_ROIMG:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_roimg_rwimg(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_ROIMG:
    case BRIG_TYPE_RWIMG:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_roimg_rwimg_woimg(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_ROIMG:
    case BRIG_TYPE_RWIMG:
    case BRIG_TYPE_WOIMG:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_woimg_rwimg(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_RWIMG:
    case BRIG_TYPE_WOIMG:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_u32(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_U32:
        return true;
    default:
        return false;
    }
}

//================================================================================
//  Req d0_s1 = {
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_d0_s1(T inst) const
{
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req d0_s1_s2 = {
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg, imm;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_d0_s1_s2(T inst) const
{
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req d0_s1_s2_s3 = {
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg, imm;
//      s3 = reg, imm;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_d0_s1_s2_s3(T inst) const
{
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req imagefence = {
//      type = none;
//      s0 = null;
//      s1 = null;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_imagefence(T inst) const
{
    if (!check_type_values_none(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_NONE, sizeof(TYPE_VALUES_NONE) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_S0, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req ld_image = {
//      ld_st_image;
//      itype = roimg, rwimg;
//      {geometry = 1d, 1db; ? d0 = vec_4; s2 = reg_ctype, imm_ctype;}
//      {geometry = 2d, 1da; ? d0 = vec_4; s2 = vec_2_ctype;}
//      {geometry = 3d, 2da; ? d0 = vec_4; s2 = vec_3_ctype;}
//      {geometry = 2ddepth; ? d0 = reg; s2 = vec_2_ctype;}
//      {geometry = 2dadepth; ? d0 = reg; s2 = vec_3_ctype;}
//      ;
//      s1 = reg_itype;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_ld_image(T inst) const
{
    req_ld_st_image(inst);
    if (!check_type_values_roimg_rwimg(getImageType<T>(inst))) {
        brigPropError(inst, PROP_IMAGETYPE, getImageType<T>(inst), TYPE_VALUES_ROIMG_RWIMG, sizeof(TYPE_VALUES_ROIMG_RWIMG) / sizeof(unsigned));
    }

    if (
            check_geometry_values_1d_1db(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_VEC4, sizeof(OPERAND_VALUES_VEC4) / sizeof(unsigned));
        validateOperand(inst, PROP_S2, OPERAND_ATTR_CTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    }
    else if (
            check_geometry_values_2d_1da(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_VEC4, sizeof(OPERAND_VALUES_VEC4) / sizeof(unsigned));
        validateOperand(inst, PROP_S2, OPERAND_ATTR_CTYPE, OPERAND_VALUES_VEC2CTYPE, sizeof(OPERAND_VALUES_VEC2CTYPE) / sizeof(unsigned));
    }
    else if (
            check_geometry_values_3d_2da(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_VEC4, sizeof(OPERAND_VALUES_VEC4) / sizeof(unsigned));
        validateOperand(inst, PROP_S2, OPERAND_ATTR_CTYPE, OPERAND_VALUES_VEC3CTYPE, sizeof(OPERAND_VALUES_VEC3CTYPE) / sizeof(unsigned));
    }
    else if (
            check_geometry_values_2ddepth(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
        validateOperand(inst, PROP_S2, OPERAND_ATTR_CTYPE, OPERAND_VALUES_VEC2CTYPE, sizeof(OPERAND_VALUES_VEC2CTYPE) / sizeof(unsigned));
    }
    else if (
            check_geometry_values_2dadepth(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
        validateOperand(inst, PROP_S2, OPERAND_ATTR_CTYPE, OPERAND_VALUES_VEC3CTYPE, sizeof(OPERAND_VALUES_VEC3CTYPE) / sizeof(unsigned));
    }
    else
    {
        invalidVariant(inst, PROP_GEOMETRY);
    }
    validateOperand(inst, PROP_S1, OPERAND_ATTR_ITYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req ld_st_image = {
//      type = s32, u32, f32, f16;
//      ctype = u32;
//      geometry = any;
//      eqclass = any;
//  }
template<class T> bool InstValidator::req_ld_st_image(T inst) const
{
    if (!check_type_values_s32_u32_f32_f16(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_S32_U32_F32_F16, sizeof(TYPE_VALUES_S32_U32_F32_F16) / sizeof(unsigned));
    }
    if (!check_type_values_u32(getCoordType<T>(inst))) {
        brigPropError(inst, PROP_COORDTYPE, getCoordType<T>(inst), TYPE_VALUES_U32, sizeof(TYPE_VALUES_U32) / sizeof(unsigned));
    }
    if (!check_geometry_values_any(getGeometry<T>(inst))) {
        brigPropError(inst, PROP_GEOMETRY, getGeometry<T>(inst), GEOMETRY_VALUES_ANY, sizeof(GEOMETRY_VALUES_ANY) / sizeof(unsigned));
    }
    validateEqclass(inst, PROP_EQUIVCLASS, EQCLASS_ATTR_NONE, EQCLASS_VALUES_ANY, sizeof(EQCLASS_VALUES_ANY) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req queryimage = {
//      type = u32;
//      itype = roimg, rwimg, woimg;
//      geometry = any;
//      iprop = any;
//      {iprop = width, channeltype, channelorder; ? geometry = any;}
//      {iprop = depth; ? geometry = 3d;}
//      {iprop = array; ? geometry = 1da, 2da, 2dadepth;}
//      {iprop = height; ? geometry = 2d, 3d, 2da, 2ddepth, 2dadepth;}
//      ;
//      d0 = reg;
//      s1 = reg_itype;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_queryimage(T inst) const
{
    if (!check_type_values_u32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_U32, sizeof(TYPE_VALUES_U32) / sizeof(unsigned));
    }
    if (!check_type_values_roimg_rwimg_woimg(getImageType<T>(inst))) {
        brigPropError(inst, PROP_IMAGETYPE, getImageType<T>(inst), TYPE_VALUES_ROIMG_RWIMG_WOIMG, sizeof(TYPE_VALUES_ROIMG_RWIMG_WOIMG) / sizeof(unsigned));
    }
    if (!check_geometry_values_any(getGeometry<T>(inst))) {
        brigPropError(inst, PROP_GEOMETRY, getGeometry<T>(inst), GEOMETRY_VALUES_ANY, sizeof(GEOMETRY_VALUES_ANY) / sizeof(unsigned));
    }
    if (!check_iprop_values_any(getImageQuery<T>(inst))) {
        brigPropError(inst, PROP_IMAGEQUERY, getImageQuery<T>(inst), IPROP_VALUES_ANY, sizeof(IPROP_VALUES_ANY) / sizeof(unsigned));
    }

    if (
            check_iprop_values_width_channeltype_channelorder(getImageQuery<T>(inst))
       )
    {
        if (!check_geometry_values_any(getGeometry<T>(inst))) {
            brigPropError(inst, PROP_GEOMETRY, getGeometry<T>(inst), GEOMETRY_VALUES_ANY, sizeof(GEOMETRY_VALUES_ANY) / sizeof(unsigned));
        }
    }
    else if (
            check_iprop_values_depth(getImageQuery<T>(inst))
       )
    {
        if (!check_geometry_values_3d(getGeometry<T>(inst))) {
            brigPropError(inst, PROP_GEOMETRY, getGeometry<T>(inst), GEOMETRY_VALUES_3D, sizeof(GEOMETRY_VALUES_3D) / sizeof(unsigned));
        }
    }
    else if (
            check_iprop_values_array(getImageQuery<T>(inst))
       )
    {
        if (!check_geometry_values_1da_2da_2dadepth(getGeometry<T>(inst))) {
            brigPropError(inst, PROP_GEOMETRY, getGeometry<T>(inst), GEOMETRY_VALUES_1DA_2DA_2DADEPTH, sizeof(GEOMETRY_VALUES_1DA_2DA_2DADEPTH) / sizeof(unsigned));
        }
    }
    else if (
            check_iprop_values_height(getImageQuery<T>(inst))
       )
    {
        if (!check_geometry_values_2d_3d_2da_2ddepth_2dadepth(getGeometry<T>(inst))) {
            brigPropError(inst, PROP_GEOMETRY, getGeometry<T>(inst), GEOMETRY_VALUES_2D_3D_2DA_2DDEPTH_2DADEPTH, sizeof(GEOMETRY_VALUES_2D_3D_2DA_2DDEPTH_2DADEPTH) / sizeof(unsigned));
        }
    }
    else
    {
        invalidVariant(inst, PROP_IMAGEQUERY);
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_ITYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req querysampler = {
//      type = u32;
//      sprop = any;
//      d0 = reg;
//      s1 = reg_samp;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_querysampler(T inst) const
{
    if (!check_type_values_u32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_U32, sizeof(TYPE_VALUES_U32) / sizeof(unsigned));
    }
    if (!check_sprop_values_any(getSamplerQuery<T>(inst))) {
        brigPropError(inst, PROP_SAMPLERQUERY, getSamplerQuery<T>(inst), SPROP_VALUES_ANY, sizeof(SPROP_VALUES_ANY) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_SAMP, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req rdimage = {
//      type = s32, u32, f32, f16;
//      itype = roimg;
//      ctype = s32, f32;
//      geometry = 1d, 2d, 3d, 1da, 2da, 2ddepth, 2dadepth;
//      eqclass = any;
//      s1 = reg_itype;
//      s2 = reg_samp;
//      s4 = null;
//      s5 = null;
//      {geometry = 1d; ? d0 = vec_4; s3 = reg_ctype, imm_ctype;}
//      {geometry = 2d, 1da; ? d0 = vec_4; s3 = vec_2_ctype;}
//      {geometry = 3d, 2da; ? d0 = vec_4; s3 = vec_3_ctype;}
//      {geometry = 2ddepth; ? d0 = reg; s3 = vec_2_ctype;}
//      {geometry = 2dadepth; ? d0 = reg; s3 = vec_3_ctype;}
//      ;
//  }
template<class T> bool InstValidator::req_rdimage(T inst) const
{
    if (!check_type_values_s32_u32_f32_f16(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_S32_U32_F32_F16, sizeof(TYPE_VALUES_S32_U32_F32_F16) / sizeof(unsigned));
    }
    if (!check_type_values_roimg(getImageType<T>(inst))) {
        brigPropError(inst, PROP_IMAGETYPE, getImageType<T>(inst), TYPE_VALUES_ROIMG, sizeof(TYPE_VALUES_ROIMG) / sizeof(unsigned));
    }
    if (!check_type_values_s32_f32(getCoordType<T>(inst))) {
        brigPropError(inst, PROP_COORDTYPE, getCoordType<T>(inst), TYPE_VALUES_S32_F32, sizeof(TYPE_VALUES_S32_F32) / sizeof(unsigned));
    }
    if (!check_geometry_values_1d_2d_3d_1da_2da_2ddepth_2dadepth(getGeometry<T>(inst))) {
        brigPropError(inst, PROP_GEOMETRY, getGeometry<T>(inst), GEOMETRY_VALUES_1D_2D_3D_1DA_2DA_2DDEPTH_2DADEPTH, sizeof(GEOMETRY_VALUES_1D_2D_3D_1DA_2DA_2DDEPTH_2DADEPTH) / sizeof(unsigned));
    }
    validateEqclass(inst, PROP_EQUIVCLASS, EQCLASS_ATTR_NONE, EQCLASS_VALUES_ANY, sizeof(EQCLASS_VALUES_ANY) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_ITYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_SAMP, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));

    if (
            check_geometry_values_1d(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_VEC4, sizeof(OPERAND_VALUES_VEC4) / sizeof(unsigned));
        validateOperand(inst, PROP_S3, OPERAND_ATTR_CTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    }
    else if (
            check_geometry_values_2d_1da(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_VEC4, sizeof(OPERAND_VALUES_VEC4) / sizeof(unsigned));
        validateOperand(inst, PROP_S3, OPERAND_ATTR_CTYPE, OPERAND_VALUES_VEC2CTYPE, sizeof(OPERAND_VALUES_VEC2CTYPE) / sizeof(unsigned));
    }
    else if (
            check_geometry_values_3d_2da(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_VEC4, sizeof(OPERAND_VALUES_VEC4) / sizeof(unsigned));
        validateOperand(inst, PROP_S3, OPERAND_ATTR_CTYPE, OPERAND_VALUES_VEC3CTYPE, sizeof(OPERAND_VALUES_VEC3CTYPE) / sizeof(unsigned));
    }
    else if (
            check_geometry_values_2ddepth(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
        validateOperand(inst, PROP_S3, OPERAND_ATTR_CTYPE, OPERAND_VALUES_VEC2CTYPE, sizeof(OPERAND_VALUES_VEC2CTYPE) / sizeof(unsigned));
    }
    else if (
            check_geometry_values_2dadepth(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
        validateOperand(inst, PROP_S3, OPERAND_ATTR_CTYPE, OPERAND_VALUES_VEC3CTYPE, sizeof(OPERAND_VALUES_VEC3CTYPE) / sizeof(unsigned));
    }
    else
    {
        invalidVariant(inst, PROP_GEOMETRY);
    }
    return true;
}

//================================================================================
//  Req st_image = {
//      ld_st_image;
//      itype = woimg, rwimg;
//      {geometry = 1d, 1db; ? s0 = vec_4; s2 = reg_ctype, imm_ctype;}
//      {geometry = 2d, 1da; ? s0 = vec_4; s2 = vec_2_ctype;}
//      {geometry = 3d, 2da; ? s0 = vec_4; s2 = vec_3_ctype;}
//      {geometry = 2ddepth; ? s0 = reg; s2 = vec_2_ctype;}
//      {geometry = 2dadepth; ? s0 = reg; s2 = vec_3_ctype;}
//      ;
//      s1 = reg_itype;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_st_image(T inst) const
{
    req_ld_st_image(inst);
    if (!check_type_values_woimg_rwimg(getImageType<T>(inst))) {
        brigPropError(inst, PROP_IMAGETYPE, getImageType<T>(inst), TYPE_VALUES_WOIMG_RWIMG, sizeof(TYPE_VALUES_WOIMG_RWIMG) / sizeof(unsigned));
    }

    if (
            check_geometry_values_1d_1db(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_S0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_VEC4, sizeof(OPERAND_VALUES_VEC4) / sizeof(unsigned));
        validateOperand(inst, PROP_S2, OPERAND_ATTR_CTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    }
    else if (
            check_geometry_values_2d_1da(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_S0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_VEC4, sizeof(OPERAND_VALUES_VEC4) / sizeof(unsigned));
        validateOperand(inst, PROP_S2, OPERAND_ATTR_CTYPE, OPERAND_VALUES_VEC2CTYPE, sizeof(OPERAND_VALUES_VEC2CTYPE) / sizeof(unsigned));
    }
    else if (
            check_geometry_values_3d_2da(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_S0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_VEC4, sizeof(OPERAND_VALUES_VEC4) / sizeof(unsigned));
        validateOperand(inst, PROP_S2, OPERAND_ATTR_CTYPE, OPERAND_VALUES_VEC3CTYPE, sizeof(OPERAND_VALUES_VEC3CTYPE) / sizeof(unsigned));
    }
    else if (
            check_geometry_values_2ddepth(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_S0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
        validateOperand(inst, PROP_S2, OPERAND_ATTR_CTYPE, OPERAND_VALUES_VEC2CTYPE, sizeof(OPERAND_VALUES_VEC2CTYPE) / sizeof(unsigned));
    }
    else if (
            check_geometry_values_2dadepth(getGeometry<T>(inst))
       )
    {
        validateOperand(inst, PROP_S0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
        validateOperand(inst, PROP_S2, OPERAND_ATTR_CTYPE, OPERAND_VALUES_VEC3CTYPE, sizeof(OPERAND_VALUES_VEC3CTYPE) / sizeof(unsigned));
    }
    else
    {
        invalidVariant(inst, PROP_GEOMETRY);
    }
    validateOperand(inst, PROP_S1, OPERAND_ATTR_ITYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

void InstValidator::validateInst(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_IMAGEFENCE):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_imagefence<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_LDIMAGE):
        {
            InstImage i = inst;
            if (!i) { invalidFormat(inst, "InstImage"); }
            req_ld_image<InstImage>(i);
            break;
        }
        case (BRIG_OPCODE_QUERYIMAGE):
        {
            InstQueryImage i = inst;
            if (!i) { invalidFormat(inst, "InstQueryImage"); }
            req_queryimage<InstQueryImage>(i);
            break;
        }
        case (BRIG_OPCODE_QUERYSAMPLER):
        {
            InstQuerySampler i = inst;
            if (!i) { invalidFormat(inst, "InstQuerySampler"); }
            req_querysampler<InstQuerySampler>(i);
            break;
        }
        case (BRIG_OPCODE_RDIMAGE):
        {
            InstImage i = inst;
            if (!i) { invalidFormat(inst, "InstImage"); }
            req_rdimage<InstImage>(i);
            break;
        }
        case (BRIG_OPCODE_STIMAGE):
        {
            InstImage i = inst;
            if (!i) { invalidFormat(inst, "InstImage"); }
            req_st_image<InstImage>(i);
            break;
        }
        default:
            validate(inst, false, "Invalid instruction opcode");
            break;
    }
} // InstValidator::validateInst

unsigned InstValidator::getOperand0Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_IMAGEFENCE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_LDIMAGE): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_QUERYIMAGE): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_QUERYSAMPLER): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_RDIMAGE): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_STIMAGE): return OPERAND_ATTR_DTYPE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand0Attr

unsigned InstValidator::getOperand1Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_IMAGEFENCE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_LDIMAGE): return OPERAND_ATTR_ITYPE;
        case (BRIG_OPCODE_QUERYIMAGE): return OPERAND_ATTR_ITYPE;
        case (BRIG_OPCODE_QUERYSAMPLER): return OPERAND_ATTR_SAMP;
        case (BRIG_OPCODE_RDIMAGE): return OPERAND_ATTR_ITYPE;
        case (BRIG_OPCODE_STIMAGE): return OPERAND_ATTR_ITYPE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand1Attr

unsigned InstValidator::getOperand2Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_IMAGEFENCE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_LDIMAGE): return OPERAND_ATTR_CTYPE;
        case (BRIG_OPCODE_QUERYIMAGE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_QUERYSAMPLER): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_RDIMAGE): return OPERAND_ATTR_SAMP;
        case (BRIG_OPCODE_STIMAGE): return OPERAND_ATTR_CTYPE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand2Attr

unsigned InstValidator::getOperand3Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_IMAGEFENCE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_LDIMAGE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_QUERYIMAGE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_QUERYSAMPLER): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_RDIMAGE): return OPERAND_ATTR_CTYPE;
        case (BRIG_OPCODE_STIMAGE): return OPERAND_ATTR_NONE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand3Attr

unsigned InstValidator::getOperand4Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_IMAGEFENCE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_LDIMAGE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_QUERYIMAGE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_QUERYSAMPLER): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_RDIMAGE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_STIMAGE): return OPERAND_ATTR_NONE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand4Attr

unsigned InstValidator::getOperand5Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_IMAGEFENCE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_LDIMAGE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_QUERYIMAGE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_QUERYSAMPLER): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_RDIMAGE): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_STIMAGE): return OPERAND_ATTR_NONE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand5Attr

